/* PS/2 mouse to Atari ST adapter. (C) 2006 - 2016 Christian Zietz <czietz@gmx.net> */
// -- PS/2 related functions

/*

Copyright (c) 2016-2016 Christian Zietz <czietz@gmx.net> 

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.

*/

#include <avr/io.h>
#include <util/parity.h>
#include "ps2.h"

#define WAIT_CLK_HI()  { while (!(PS2PIN & (1<<CLK))) ; }
#define WAIT_CLK_LOW() { while (PS2PIN & (1<<CLK)) ; }
#define CLK_HI         { PS2PORT |= (1<<CLK); }
#define CLK_LOW        { PS2PORT &= ~(1<<CLK); }

#define IN  0
#define OUT 1

#define CLKDIR(x)      { if (x) PS2DDR |= (1<<CLK); else PS2DDR &= ~(1<<CLK); }
#define DATADIR(x)     { if (x) PS2DDR |= (1<<DATA); else PS2DDR &= ~(1<<DATA); }

#define DATABIT_IN     ((PS2PIN&(1<<DATA))?1:0)
#define DATABIT_OUT(x) { if (x) PS2PORT |= (1<<DATA); else PS2PORT &= ~(1<<DATA); } 

#define CLK_PULLUP     { CLKDIR(IN); CLK_HI; }
#define DATA_PULLUP    { DATADIR(IN); DATABIT_OUT(1); }

void delay(long x) {
  volatile long y = x;
  while (y--) ;
}

int ps2recv(void) {
  unsigned char x = 0;
  char i;
  
  CLK_PULLUP
  DATA_PULLUP

  WAIT_CLK_HI();
  
  // start bit
  WAIT_CLK_LOW();
  if (DATABIT_IN) {
    return -1;
  }
  WAIT_CLK_HI();
  
  for (i=0; i<8; i++) {
    x = x >> 1;
    WAIT_CLK_LOW();
    x |= DATABIT_IN << 7;
    WAIT_CLK_HI();
  }

  // parity
  WAIT_CLK_LOW();
  WAIT_CLK_HI();
  
  // stop bit
  WAIT_CLK_LOW();
  if (!DATABIT_IN) {
    return -1;
  }
  WAIT_CLK_HI();
 
  return x;
}

int ps2send(unsigned char d) {
  char i, p;
  
  CLK_LOW
  CLKDIR(OUT);
  p = 1-parity_even_bit(d);
  delay(400);
  
  DATABIT_OUT(0);  
  DATADIR(OUT);
  
  CLK_PULLUP
  
  delay(5);
  WAIT_CLK_LOW();
  
  for (i=0; i<8; i++) {
    DATABIT_OUT(d & 1);
    d = d >> 1;
    WAIT_CLK_HI();
    WAIT_CLK_LOW();
  }
    
  DATABIT_OUT(p);
  WAIT_CLK_HI();
  WAIT_CLK_LOW();
  
  DATA_PULLUP
  delay(5);
  while (DATABIT_IN) {}
  WAIT_CLK_LOW();
  WAIT_CLK_HI();
  
  return 0;
}
